@@ -34,6 +34,7 @@
 #include "gmime-filter-crlf.h"
 #include "gmime-stream-mem.h"
 #include "gmime-parser.h"
+#include "gmime-error.h"
 #include "gmime-part.h"
 
 #define d(x)
@@ -57,11 +58,11 @@ static void g_mime_multipart_signed_init (GMimeMultipartSigned *mps, GMimeMultip
 static void g_mime_multipart_signed_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void multipart_signed_init (GMimeObject *object);
-static void multipart_signed_add_header (GMimeObject *object, const char *header, const char *value);
+static void multipart_signed_prepend_header (GMimeObject *object, const char *header, const char *value);
+static void multipart_signed_append_header (GMimeObject *object, const char *header, const char *value);
 static void multipart_signed_set_header (GMimeObject *object, const char *header, const char *value);
 static const char *multipart_signed_get_header (GMimeObject *object, const char *header);
-static void multipart_signed_remove_header (GMimeObject *object, const char *header);
+static gboolean multipart_signed_remove_header (GMimeObject *object, const char *header);
 static void multipart_signed_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *multipart_signed_get_headers (GMimeObject *object);
 static ssize_t multipart_signed_write_to_stream (GMimeObject *object, GMimeStream *stream);
@@ -105,11 +106,11 @@ g_mime_multipart_signed_class_init (GMimeMultipartSignedClass *klass)
 	
 	gobject_class->finalize = g_mime_multipart_signed_finalize;
 	
-	object_class->init = multipart_signed_init;
-	object_class->add_header = multipart_signed_add_header;
+	object_class->prepend_header = multipart_signed_prepend_header;
+	object_class->append_header = multipart_signed_append_header;
+	object_class->remove_header = multipart_signed_remove_header;
 	object_class->set_header = multipart_signed_set_header;
 	object_class->get_header = multipart_signed_get_header;
-	object_class->remove_header = multipart_signed_remove_header;
 	object_class->set_content_type = multipart_signed_set_content_type;
 	object_class->get_headers = multipart_signed_get_headers;
 	object_class->write_to_stream = multipart_signed_write_to_stream;
@@ -133,18 +134,16 @@ g_mime_multipart_signed_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
 static void
-multipart_signed_init (GMimeObject *object)
+multipart_signed_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
+	GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
 }
 
 static void
-multipart_signed_add_header (GMimeObject *object, const char *header, const char *value)
+multipart_signed_append_header (GMimeObject *object, const char *header, const char *value)
 {
-	GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
+	GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
 }
 
 static void
@@ -159,10 +158,10 @@ multipart_signed_get_header (GMimeObject *object, const char *header)
 	return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 }
 
-static void
+static gboolean
 multipart_signed_remove_header (GMimeObject *object, const char *header)
 {
-	GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
 static void
@@ -200,7 +199,7 @@ multipart_signed_write_to_stream (GMimeObject *object, GMimeStream *stream)
  *
  * Creates a new MIME multipart/signed object.
  *
- * Returns an empty MIME multipart/signed object.
+ * Returns: an empty MIME multipart/signed object.
  **/
 GMimeMultipartSigned *
 g_mime_multipart_signed_new (void)
@@ -229,21 +228,24 @@ static void
 sign_prepare (GMimeObject *mime_part)
 {
 	GMimeContentEncoding encoding;
+	GMimeMultipart *multipart;
 	GMimeObject *subpart;
+	int i, n;
 	
 	if (GMIME_IS_MULTIPART (mime_part)) {
-		GList *lpart;
+		multipart = (GMimeMultipart *) mime_part;
 		
-		if (GMIME_IS_MULTIPART_SIGNED (mime_part) || GMIME_IS_MULTIPART_ENCRYPTED (mime_part)) {
+		if (GMIME_IS_MULTIPART_SIGNED (multipart) ||
+		    GMIME_IS_MULTIPART_ENCRYPTED (multipart)) {
 			/* must not modify these parts as they must be treated as opaque */
 			return;
 		}
 		
-		lpart = GMIME_MULTIPART (mime_part)->subparts;
-		while (lpart) {
-			subpart = GMIME_OBJECT (lpart->data);
+		n = g_mime_multipart_get_number (multipart);
+		for (i = 0; i < n; i++) {
+			subpart = g_mime_multipart_get_part (multipart, i);
 			sign_prepare (subpart);
-			lpart = lpart->next;
+			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (mime_part)) {
 		subpart = GMIME_MESSAGE_PART (mime_part)->message->mime_part;
@@ -272,7 +274,7 @@ sign_prepare (GMimeObject *mime_part)
  * successful, the signed #GMimeObject is set as the signed part of
  * the multipart/signed object @mps.
  *
- * Returns %0 on success or %-1 on fail. If the signing fails, an
+ * Returns: %0 on success or %-1 on fail. If the signing fails, an
  * exception will be set on @err to provide information as to why the
  * failure occured.
  **/
@@ -281,13 +283,13 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 			      GMimeCipherContext *ctx, const char *userid,
 			      GMimeCipherHash hash, GError **err)
 {
-	GMimeObject *signature;
-	GMimeDataWrapper *wrapper;
-	GMimeStream *filtered_stream;
-	GMimeStream *stream, *sigstream;
-	GMimeFilter *crlf_filter, *from_filter, *strip_filter;
+	GMimeStream *stream, *filtered, *sigstream;
 	GMimeContentType *content_type;
+	GMimeDataWrapper *wrapper;
+	GMimeObject *signature;
+	GMimeFilter *filter;
 	GMimeParser *parser;
+	int rv;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), -1);
 	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), -1);
@@ -299,42 +301,41 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	
 	/* get the cleartext */
 	stream = g_mime_stream_mem_new ();
-	filtered_stream = g_mime_stream_filter_new_with_stream (stream);
+	filtered = g_mime_stream_filter_new (stream);
 	
 	/* Note: see rfc3156, section 3 - second note */
-	from_filter = g_mime_filter_from_new (GMIME_FILTER_FROM_MODE_ARMOR);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), from_filter);
-	g_object_unref (from_filter);
+	filter = g_mime_filter_from_new (GMIME_FILTER_FROM_MODE_ARMOR);
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_object_unref (filter);
 	
 	/* Note: see rfc3156, section 5.4 (this is the main difference between rfc2015 and rfc3156) */
-	strip_filter = g_mime_filter_strip_new ();
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), strip_filter);
-	g_object_unref (strip_filter);
+	filter = g_mime_filter_strip_new ();
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_object_unref (filter);
 	
-	g_mime_object_write_to_stream (content, filtered_stream);
-	g_mime_stream_flush (filtered_stream);
-	g_object_unref (filtered_stream);
+	g_mime_object_write_to_stream (content, filtered);
+	g_mime_stream_flush (filtered);
+	g_object_unref (filtered);
 	g_mime_stream_reset (stream);
 	
 	/* Note: see rfc2015 or rfc3156, section 5.1 */
-	filtered_stream = g_mime_stream_filter_new_with_stream (stream);
-	crlf_filter = g_mime_filter_crlf_new (GMIME_FILTER_CRLF_ENCODE,
-					      GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), crlf_filter);
-	g_object_unref (crlf_filter);
+	filtered = g_mime_stream_filter_new (stream);
+	filter = g_mime_filter_crlf_new (TRUE, FALSE);
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_object_unref (filter);
 	
 	/* construct the signature stream */
 	sigstream = g_mime_stream_mem_new ();
 	
 	/* sign the content stream */
-	if (g_mime_cipher_context_sign (ctx, userid, hash, filtered_stream, sigstream, err) == -1) {
-		g_object_unref (filtered_stream);
+	if ((rv = g_mime_cipher_context_sign (ctx, userid, hash, filtered, sigstream, err)) == -1) {
 		g_object_unref (sigstream);
+		g_object_unref (filtered);
 		g_object_unref (stream);
 		return -1;
 	}
 	
-	g_object_unref (filtered_stream);
+	g_object_unref (filtered);
 	g_mime_stream_reset (sigstream);
 	g_mime_stream_reset (stream);
 	
@@ -353,7 +354,7 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_object_unref (wrapper);
 	
 	mps->protocol = g_strdup (ctx->sign_protocol);
-	mps->micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, hash));
+	mps->micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, (GMimeCipherHash) rv));
 	
 	/* set the content-type of the signature part */
 	content_type = g_mime_content_type_new_from_string (mps->protocol);
@@ -392,8 +393,8 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
  * Attempts to verify the signed MIME part contained within the
  * multipart/signed object @mps using the @ctx cipher context.
  *
- * Returns a new #GMimeCipherValidity object on success or %NULL on
- * fail. If the signing fails, an exception will be set on @err to
+ * Returns: a new #GMimeSignatureValidity object on success or %NULL
+ * on fail. If the signing fails, an exception will be set on @err to
  * provide information as to why the failure occured.
  **/
 GMimeSignatureValidity *
@@ -414,8 +415,9 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (ctx->sign_protocol != NULL, NULL);
 	
-	if (g_mime_multipart_get_number ((GMimeMultipart *) mps) != 2) {
-		/* FIXME: set an exception */
+	if (g_mime_multipart_get_number ((GMimeMultipart *) mps) < 2) {
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
+			     "Cannot verify multipart/signed part due to missing subparts.");
 		return NULL;
 	}
 	
@@ -447,11 +449,10 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	
 	/* get the content stream */
 	stream = g_mime_stream_mem_new ();
-	filtered_stream = g_mime_stream_filter_new_with_stream (stream);
+	filtered_stream = g_mime_stream_filter_new (stream);
 	
 	/* Note: see rfc2015 or rfc3156, section 5.1 */
-	crlf_filter = g_mime_filter_crlf_new (GMIME_FILTER_CRLF_ENCODE,
-					      GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+	crlf_filter = g_mime_filter_crlf_new (TRUE, FALSE);
 	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), crlf_filter);
 	g_object_unref (crlf_filter);
 	
